﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace SqliteUsingSQLiteWinRT.ViewModels
{
  public class CustomersViewModel : TableViewModelBase<CustomerViewModel, int>
  {
    private CustomersViewModel() { }

    static CustomersViewModel instance;
    public static CustomersViewModel GetDefault()
    {
      lock (typeof(CustomersViewModel))
      {
        if (instance == null)
          instance = new CustomersViewModel();
      }

      return instance;
    }

    protected override string GetSelectAllSql()
    {
      return "SELECT Id, Name, City, Contact FROM Customer";
    }

    protected override void FillSelectAllStatement(SQLiteWinRT.Statement statement)
    {
      // nothing to do
    }

    protected override CustomerViewModel CreateItem(SQLiteWinRT.Statement statement)
    {
      var c = new CustomerViewModel(
        statement.GetIntAt(0),
        statement.GetTextAt(1),
        statement.GetTextAt(2),
        statement.GetTextAt(3));
      Debug.WriteLine("Selected Customer name:" + c.Name);

      return c;
    }

    protected override string GetSelectItemSql()
    {
      return "SELECT Id, Name, City, Contact FROM Customer WHERE Id = ?";
    }

    protected override void FillSelectItemStatement(SQLiteWinRT.Statement statement, int key)
    {
      statement.BindIntParameterAt(1, key);
    }

    protected override string GetInsertItemSql()
    {
      return "INSERT INTO Customer (Name, City, Contact) VALUES (@name, @city, @contact)";
    }

    protected override void FillInsertStatement(SQLiteWinRT.Statement statement, CustomerViewModel item)
    {
      // NOTE that named parameters have a leading "@",":" or "$".
      statement.BindTextParameterWithName("@name", item.Name);
      statement.BindTextParameterWithName("@city", item.City);
      statement.BindTextParameterWithName("@contact", item.Contact);
    }

    protected override string GetUpdateItemSql()
    {
      return "UPDATE Customer SET Name = ?, City = ?, Contact = ? WHERE Id = ?";
    }

    protected override void FillUpdateStatement(SQLiteWinRT.Statement statement, int key, CustomerViewModel item)
    {
      // NOTE that the first host parameter has an index of 1, not 0.
      statement.BindTextParameterAt(1, item.Name);
      statement.BindTextParameterAt(2, item.City);
      statement.BindTextParameterAt(3, item.Contact);
      statement.BindIntParameterAt(4, key);
    }

    protected override string GetDeleteItemSql()
    {
      return "DELETE FROM Customer WHERE Id = ?";
    }

    protected override void FillDeleteItemStatement(SQLiteWinRT.Statement statement, int key)
    {
      statement.BindIntParameterAt(1, key);
    }
  }
}
