﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using SqliteUsingSQLiteWinRT.Resources;
using SqliteUsingSQLiteWinRT.ViewModels;
using System.Runtime.InteropServices;
using System.Threading.Tasks;

namespace SqliteUsingSQLiteWinRT.Views
{
  public partial class CustomerEditView : PhoneApplicationPage
  {
    private CustomerViewModel viewModel;
    DateTime timestamp = DateTime.MinValue;

    public CustomerEditView()
    {
      InitializeComponent();
    }

    protected async override void OnNavigatedTo(NavigationEventArgs e)
    {
      if (viewModel != null && timestamp >= ProjectsViewModel.GetDefault().Timestamp)
        return;

      string value;
      if (NavigationContext.QueryString.TryGetValue("NEW", out value))
        InitForNewCustomer();
      else if (NavigationContext.QueryString.TryGetValue("ID", out value))
        await InitForEditCustomer(Int32.Parse(value));

      this.DataContext = viewModel;
    }

    private async Task InitForEditCustomer(int ID)
    {
      this.PageTitle.Text = AppResources.EditCustomerPageTitle;
      viewModel = await CustomersViewModel.GetDefault().GetItemAsync(ID);
    }

    private void InitForNewCustomer()
    {
      this.PageTitle.Text = AppResources.NewCustomerPageTitle;
      viewModel = new CustomerViewModel();
    }

    private async void SaveButton_Click(object sender, EventArgs e)
    {
      ForceTextBoxDataBindingUpdate();

      try
      {
        if (viewModel.IsNew)
          await CustomersViewModel.GetDefault().InsertItemAsync(viewModel);
        else if (viewModel.IsDirty)
          await CustomersViewModel.GetDefault().UpdateItemAsync(viewModel.Id, viewModel);

        if (NavigationService.CanGoBack)
          NavigationService.GoBack();
        else
          DataContext = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show("Can't save that item: " + ex.Message + ". " + SQLiteWinRT.Database.GetSqliteErrorCode(ex.HResult));
      }
    }

    private void CancelButton_Click(object sender, EventArgs e)
    {
      if (NavigationService.CanGoBack)
        NavigationService.GoBack();
    }

    private void ForceTextBoxDataBindingUpdate()
    {
      object focusObj = System.Windows.Input.FocusManager.GetFocusedElement();

      if (focusObj != null && focusObj is TextBox)
      {
        var binding = (focusObj as TextBox).GetBindingExpression(TextBox.TextProperty);
        binding.UpdateSource();
      }
    }
  }
}