﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using SqliteUsingSQLiteWinRT.ViewModels;
using System.Runtime.InteropServices;

namespace SqliteUsingSQLiteWinRT.Views
{
  public partial class CustomerProjectsView : PhoneApplicationPage
  {
    private CustomerViewModel customer;
    private ProjectsViewModel projectsViewModel;
    DateTime timestamp = DateTime.MinValue;

    public CustomerProjectsView()
    {
      InitializeComponent();
    }

    protected async override void OnNavigatedTo(NavigationEventArgs e)
    {
      if (projectsViewModel != null && timestamp >= projectsViewModel.Timestamp && timestamp >= CustomersViewModel.GetDefault().Timestamp)
        return;

      string value;
      if (!NavigationContext.QueryString.TryGetValue("ID", out value))
        throw new InvalidOperationException("Must supply an Id");

      try
      {
        customer = await CustomersViewModel.GetDefault().GetItemAsync(Int32.Parse(value));
        this.DataContext = customer;

        try
        {
          projectsViewModel = ProjectsViewModel.GetForCustomerId(customer.Id);
          ProjectsList.ItemsSource = await projectsViewModel.GetAllItems();
          ProjectsList.SelectedItem = null;
        }
        catch (Exception ex)
        {
          MessageBox.Show("Can't get projects for this customer: " + ex.Message + ". " + SQLiteWinRT.Database.GetSqliteErrorCode(ex.HResult));
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show("Can't get details for this customer: " + ex.Message + ". " + SQLiteWinRT.Database.GetSqliteErrorCode(ex.HResult));
      }

      timestamp = DateTime.Now;
    }

    private void EditButton_Click(object sender, EventArgs e)
    {
      NavigationService.Navigate(new Uri("/Views/CustomerEditView.xaml?ID=" + customer.Id, UriKind.Relative));
    }

    private async void DeleteButton_Click(object sender, EventArgs e)
    {
      try
      {
        await CustomersViewModel.GetDefault().DeleteItemAsync(customer.Id);
        if (NavigationService.CanGoBack)
          NavigationService.GoBack();
        else
          DataContext = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show("Can't delete that item: " + ex.Message + ". " + SQLiteWinRT.Database.GetSqliteErrorCode(ex.HResult));
      }
    }

    private void Pivot_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (DetailsPivot.SelectedIndex != 0)
      {
        (this.ApplicationBar.Buttons[0] as ApplicationBarIconButton).IsEnabled = false;
        (this.ApplicationBar.Buttons[1] as ApplicationBarIconButton).IsEnabled = false;
        (this.ApplicationBar.Buttons[2] as ApplicationBarIconButton).IsEnabled = true;
      }
      else
      {
        (this.ApplicationBar.Buttons[0] as ApplicationBarIconButton).IsEnabled = true;
        (this.ApplicationBar.Buttons[1] as ApplicationBarIconButton).IsEnabled = true;
        (this.ApplicationBar.Buttons[2] as ApplicationBarIconButton).IsEnabled = false;
      }
    }

    private void ProjectsList_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (ProjectsList.SelectedItem != null)
      {
        NavigationService.Navigate(new Uri("/Views/ProjectView.xaml?ID="
            + ((ProjectViewModel)ProjectsList.SelectedItem).Id, UriKind.Relative));

        ProjectsList.SelectedItem = null;
      }
    }

    private void AddProjectButton_Click(object sender, EventArgs e)
    {
      NavigationService.Navigate(new Uri("/Views/ProjectEditView.xaml?NEW=true&CUSTOMERID=" + customer.Id, UriKind.Relative));
    }
  }
}