﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using SqliteUsingSQLiteWinRT.ViewModels;
using SqliteUsingSQLiteWinRT.Resources;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace SqliteUsingSQLiteWinRT.Views
{
  public partial class ProjectEditView : PhoneApplicationPage
  {
    private ProjectViewModel viewModel;

    public ProjectEditView()
    {
      InitializeComponent();
    }

    protected async override void OnNavigatedTo(NavigationEventArgs e)
    {
      if (viewModel != null)
        return;

      string value;
      if (NavigationContext.QueryString.TryGetValue("NEW", out value))
      {
        SetTitleForNew();
        viewModel = new ProjectViewModel(Int32.Parse(NavigationContext.QueryString["CUSTOMERID"]));
      }
      else if (NavigationContext.QueryString.TryGetValue("ID", out value))
      {
        SetTitleForEdit();
        viewModel = await ProjectsViewModel.GetDefault().GetItemAsync(Int32.Parse(value));
      }

      DataContext = viewModel;
    }

    private void SetTitleForEdit()
    {
      PageTitle.Text = AppResources.EditProjectPageTitle;
    }

    private void SetTitleForNew()
    {
      PageTitle.Text = AppResources.NewProjectPageTitle;
    }

    private async void SaveButton_Click(object sender, EventArgs e)
    {
      ForceTextBoxDataBindingUpdate();

      try
      {
        if (viewModel.IsNew)
          await ProjectsViewModel.GetDefault().InsertItemAsync(viewModel);
        else if (viewModel.IsDirty)
          await ProjectsViewModel.GetDefault().UpdateItemAsync(viewModel.Id, viewModel);

        if (NavigationService.CanGoBack)
          NavigationService.GoBack();
        else
          DataContext = null;
      }
      catch (Exception ex)
      {
        MessageBox.Show("Can't save that item: " + ex.Message + ". " + SQLiteWinRT.Database.GetSqliteErrorCode(ex.HResult));
      }
    }

    private void CancelButton_Click(object sender, EventArgs e)
    {
      if (NavigationService.CanGoBack)
        NavigationService.GoBack();
      else
        DataContext = null;
    }

    private void ForceTextBoxDataBindingUpdate()
    {
      object focusObj = System.Windows.Input.FocusManager.GetFocusedElement();

      if (focusObj != null && focusObj is TextBox)
      {
        var binding = (focusObj as TextBox).GetBindingExpression(TextBox.TextProperty);
        binding.UpdateSource();
      }
    }
  }
}